# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateResp2LevelMVMixedResponsemvu(Template):
    'Fits models with multivariate mixed response models (Normal, binary, ordered, or unordered), allowing for missing data and a choice of uniform or Wishart priors and with the responses present at 2 levels. Creates custom C code.'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level', 'Normal', 'Binary', 'Ordered multinomial', 'Unordered multinomial', 'Multivariate response']
    engines = ['CustomC']

    inputs = '''
y1 = DataMatrix('responses: ')
L2ID = IDVector('Level 2 ID: ')

for i in range(0, len(y1)):
    context['bin1_'+str(i+1)] = Text('Response Type for response ' + y1[i] + ': ', ['Normal', 'Binary', 'Ordered', 'Unordered'])
    if(context['bin1_'+str(i+1)] == 'Ordered'):
       context['ncats'+y1[i]] = Integer('Number of categories (numbered 0,1..) for ' + y1[i] + ': ')
    if(context['bin1_'+str(i+1)] == 'Unordered'):
       context['ncats'+y1[i]] = Integer('Number of categories (numbered 0,1..) for ' + y1[i] + ': ')
       for j in range(0, context['ncats'+y1[i]]-1):
          context['x1_'+str(i+1)+'_'+str(j)] = DataMatrix('Explanatory variables for category ' + str(j) + ': ', allow_cat = True)
    else:
       context['x1_'+str(i+1)] = DataMatrix('Explanatory variables for response ' + y1[i] + ': ', allow_cat = True)

anyl2resp = Boolean('Do you have any Level 2 responses?')
if anyl2resp:
    L2Data = DataFileChooser('Level 2 data set')
    y2 = DataMatrix('Responses at level 2: ', data_src=L2Data)
    for i in range(0, len(y2)):
        context['bin2_'+str(i+1)] = Text('Response Type for response ' + y2[i] + ': ', ['Normal', 'Binary', 'Ordered', 'Unordered'])
        if(context['bin2_'+str(i+1)] == 'Ordered'):
           context['ncats'+y2[i]] = Integer('Number of categories (numbered 0,1..) for ' + y2[i] + ': ')
        if(context['bin2_'+str(i+1)] == 'Unordered'):
           context['ncats'+y2[i]] = Integer('Number of categories (numbered 0,1..) for ' + y2[i] + ': ')
           for j in range(0, context['ncats'+y2[i]]-1):
              context['x2_'+str(i+1)+'_'+str(j)] = DataMatrix('Explanatory variables for category ' + str(j) + ': ', data_src=L2Data, allow_cat = True)
        else:
           context['x2_'+str(i+1)] = DataMatrix('Explanatory variables for response ' + y2[i] + ': ', data_src=L2Data, allow_cat = True)

condmarg = Boolean('Use conditional marginal algorithm: ')
deviance = ParamScalar(modelled = False)

storeresid = Boolean('Store residuals?')

lenbeta1 = 0
n1 = 0
uind = 0
for i in range(0, len(y1)):
    if(context['bin1_'+str(i+1)] == 'Normal'):
       context['miss'+y1[i]] = ParamVector(monitor=False)
       if storeresid:
           context['u' + str(uind)] = ParamVector(parents=[L2ID], as_scalar=False)
       else:
           context['u' + str(uind)] = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)
    if(context['bin1_'+str(i+1)] == 'Binary'):
       context['misb'+y1[i]] = ParamVector(monitor=False)
       if storeresid:
           context['u' + str(uind)] = ParamVector(parents=[L2ID], as_scalar=False)
       else:
           context['u' + str(uind)] = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)
    if(context['bin1_'+str(i+1)] == 'Ordered'):
       context['miso'+y1[i]] = ParamVector(monitor=False)
       context['alpha'+y1[i]] = ParamVector()
       context['alpha'+y1[i]].ncols = -(context['ncats'+y1[i]] -1)
       context['alpha'+y1[i]].set_value( map(str, range(0, (context['ncats'+y1[i]] -1))) )
       if storeresid:
           context['u1_' + str(uind)] = ParamVector(parents=[L2ID], as_scalar=False)
       else:
           context['u1_' + str(uind)] = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)
    if(context['bin1_'+str(i+1)] == 'Unordered'):
       for j in range(0, context['ncats'+y1[i]]-1):
          lenbeta1 += len(context['x1_'+str(i+1)+'_'+str(j)])
          context['misu'+str(context['ncats'+y1[i]])+'_'+y1[i]+'_'+ str(j)] = ParamVector(monitor=False)
          if storeresid:
              context['u1_' + str(uind)] = ParamVector(parents=[L2ID], as_scalar=False)
          else:
              context['u1_' + str(uind)] = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)
          uind += 1
       context['y1_'+ str(i+1)] = DataMatrix(value=[])
       context['origv1_'+str(i+1)] = List(value = [])
       n1 += context['ncats'+y1[i]] - 1
    else:
       lenbeta1 += len(context['x1_'+str(i+1)])
       n1 += 1
       if storeresid:
           context['u1_' + str(uind)] = ParamVector(parents=[L2ID], as_scalar=False)
       else:
           context['u1_' + str(uind)] = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)
       uind += 1

omega_e1 = ParamMatrix()
omega_e1.size = n1
omega_e1.select('Starting values for Level 1 variance matrix (lower triangle): ')
# omega_u1 = ParamMatrix()
# omega_u1.size = n1
d_e1 = ParamMatrix(modelled = False)
d_e1.size = n1
# d_u1 = ParamMatrix()
# d_u1.size = n1
beta1 = ParamVector()
beta1.ncols = lenbeta1

if anyl2resp:
    lenbeta2 = 0
    n2 = 0
    for i in range(0, len(y2)):
        if(context['bin2_'+str(i+1)] == 'Normal'):
           context['miss'+str(L2Data)+'_'+y2[i]] = ParamVector(monitor=False)
        if(context['bin2_'+str(i+1)] == 'Binary'):
           context['misb'+str(L2Data)+'_'+y2[i]] = ParamVector(monitor=False)
        if(context['bin2_'+str(i+1)] == 'Ordered'):
           context['miso'+str(L2Data)+'_'+y2[i]] = ParamVector(monitor=False)
           context['alpha'+y2[i]] = ParamVector()
           context['alpha'+y2[i]].ncols = -(context['ncats'+y2[i]] -1)
           context['alpha'+y2[i]].set_value( map(str, range(0, (context['ncats'+y2[i]] -1))) )
        if(context['bin2_'+str(i+1)] == 'Unordered'):
           for j in range(0, context['ncats'+y2[i]]-1):
              lenbeta2 += len(context['x2_'+str(i+1)+'_'+str(j)])
              context['misu'+str(L2Data)+'_'+str(context['ncats'+y2[i]])+'_'+y2[i]+'_'+ str(j)] = ParamVector(monitor=False)
           context['y2_'+ str(i+1)] = DataMatrix(value=[], data_src=L2Data)
           context['origv2_'+str(i+1)] = List(value = [])
           n2 += context['ncats'+y2[i]] - 1
        else:
           lenbeta2 += len(context['x2_'+str(i+1)])
           n2 += 1

    beta2 = ParamVector()
    beta2.ncols = lenbeta2
else:
    n2 = 0
omega_u = ParamMatrix(modelled = False)
omega_u.size = n2 + n1

d_u = ParamMatrix()
d_u.size = n2 + n1
'''

    customccode = '''
        <%!
    def mmult2(names, var, index,count):
        out = ""
        first = True
        for name in names:
            if first == False:
                out += ' + '
            else:
                first = False
            out += name + '[' + index + '] * ' + var + '[' + str(count) + ']'
            count += 1
        return out

    def PyToC(var):
        if isinstance(var, list):
            return "{" + ",".join(map(PyToC, var)) + "}"
        else:
            return str(var)

    def mmult3(dset, names, var, index,count):
        out = ""
        first = True
        for name in names:
            if first == False:
                out += ' + '
            else:
                first = False
            out += str(dset) + '_' + name + '[' + index + '] * ' + var + '[' + str(count) + ']'
            count += 1
        return out
%>

<%
nvar1 = len(y1)
n1 = 0
for i in range(0, nvar1):
    if (context['bin1_'+str(i+1)] == 'Unordered'):
        n1 += context['ncats'+y1[i]] - 1
    else:
        n1 += 1

nvar2 = 0
if anyl2resp:
    nvar2 = len(y2)
n2 = 0
for i in range(0, nvar2):
    if (context['bin2_'+str(i+1)] == 'Unordered'):
        n2 += context['ncats'+y2[i]] - 1
    else:
        n2 += 1

nr2 = n2 + n1 # nr2 is total number of random effects at level 2

lenbeta1 = 0
for i in range(0, nvar1):
    if (context['bin1_'+str(i+1)] == 'Unordered'):
        for j in range(0,context['ncats'+y1[i]]-1):
            lenbeta1 += len(context['x1_' + str(i + 1) + '_'+ str(j)])
    else:
        lenbeta1 += len(context['x1_' + str(i + 1)])

lenbeta2 = 0
for i in range(0, nvar2):
    if (context['bin2_'+str(i+1)] == 'Unordered'):
        for j in range(0,context['ncats'+y2[i]]-1):
            lenbeta2 += len(context['x2_' + str(i + 1) + '_'+ str(j)])
    else:
        lenbeta2 += len(context['x2_' + str(i + 1)])

allnormal1 = True
for i in range(0, nvar1):
    if (context['bin1_'+str(i+1)] != 'Normal'):
        allnormal1 = False

allnormal2 = True
for i in range(0, nvar2):
    if (context['bin2_'+str(i+1)] != 'Normal'):
        allnormal2 = False

pyyind1 = []
pyymap1 = []
pypos1 = 0
for i in range(0, len(y1)):
    if (context['bin1_'+str(i+1)] == 'Unordered'):
        pyymap1.append(pypos1)
        for j in range(0,context['ncats'+y1[i]]-1):
            pypos1 += 1
            pyyind1.append(i)
    else:
        pyymap1.append(pypos1)
        pypos1 += 1
        pyyind1.append(i)

pyyind2 = []
pyymap2 = []
pypos2 = 0
for i in range(0, nvar2):
    if (context['bin2_'+str(i+1)] == 'Unordered'):
        pyymap2.append(pypos2)
        for j in range(0,context['ncats'+y2[i]]-1):
            pypos2 += 1
            pyyind2.append(i)
    else:
        pyymap2.append(pypos2)
        pypos2 += 1
        pyyind2.append(i)

%>

                std::vector<int> betaind1(${lenbeta1});
<% betacount = 0 %>
<% row = 0 %>\\
% for i in range(0, len(y1)):
% if (context['bin1_'+str(i+1)] == 'Unordered'):
% for j in range(0, context['ncats'+y1[i]]-1):
% for k in range(0, len(context['x1_'+str(i+1)+'_'+str(j)])):
                betaind1[${betacount}] = ${row};
<% betacount += 1 %>
% endfor
<% row += 1 %>
% endfor
% else:
% for j in range(0, len(context['x1_' +str(i+1)])):
                betaind1[${betacount}] = ${row};
<% betacount += 1 %>
% endfor
<% row += 1 %>
% endif
% endfor

                std::vector<int> betaind2(${lenbeta2});
<% betacount = 0 %>
<% row = 0 %>\\
% for i in range(0, nvar2):
% if (context['bin2_'+str(i+1)] == 'Unordered'):
% for j in range(0, context['ncats'+y2[i]]-1):
% for k in range(0, len(context['x2_'+str(i+1)+'_'+str(j)])):
                betaind2[${betacount}] = ${row};
<% betacount += 1 %>
% endfor
<% row += 1 %>
% endfor
% else:
% for j in range(0, len(context['x2_' +str(i+1)])):
                betaind2[${betacount}] = ${row};
<% betacount += 1 %>
% endfor
<% row += 1 %>
% endif
% endfor

                std::vector<int> betamap1(${n1});
                int pos1 = 0;
<% ycount = 0 %>
% for i in range(0, len(y1)):
% if (context['bin1_'+str(i+1)] == 'Unordered'):
% for j in range(0, context['ncats'+y1[i]]-1):
                betamap1[${ycount}] = pos1;
                pos1 += mat_x1_${i + 1}_${j}.cols();
<% ycount += 1 %>
% endfor
% else:
                betamap1[${ycount}] = pos1;
                pos1 += mat_x1_${i + 1}.cols();
<% ycount += 1 %>
% endif
% endfor

                std::vector<int> betamap2(${n2});
                int pos2 = 0;
<% ycount = 0 %>
% for i in range(0, nvar2):
% if (context['bin2_'+str(i+1)] == 'Unordered'):
% for j in range(0, context['ncats'+y2[i]]-1):
                betamap2[${ycount}] = pos2;
                pos2 += mat_x2_${i + 1}_${j}.cols();
<% ycount += 1 %>
% endfor
% else:
                betamap2[${ycount}] = pos2;
                pos2 += mat_x2_${i + 1}.cols();
<% ycount += 1 %>
% endif
% endfor

                std::vector<double*> tmp_x1;
% for i in range(0, len(y1)):
% if (context['bin1_'+str(i+1)] == 'Unordered'):
% for j in range(0, context['ncats'+y1[i]]-1):
% for k in range(0, len(context['x1_'+str(i+1)+'_'+str(j)])):
                tmp_x1.push_back(const_cast<double *>(${context['x1_' + str(i+1)+'_'+str(j)][k]}));
% endfor
% endfor
% else:
% for j in range(0, len(context['x1_' + str(i+1)])):
                tmp_x1.push_back(const_cast<double *>(${context['x1_' + str(i+1)][j]}));
% endfor
% endif
% endfor
                RectMatrix mat_x1(tmp_x1, length(${y1[0]}));

                std::vector<double*> tmp_x2;
% for i in range(0, nvar2):
% if (context['bin2_'+str(i+1)] == 'Unordered'):
% for j in range(0, context['ncats'+y2[i]]-1):
% for k in range(0, len(context['x2_'+str(i+1)+'_'+str(j)])):
                tmp_x2.push_back(const_cast<double *>(${L2Data}_${context['x2_' + str(i+1)+'_'+str(j)][k]}));
% endfor
% endfor
% else:
% for j in range(0, len(context['x2_' + str(i+1)])):
                tmp_x2.push_back(const_cast<double *>(${L2Data}_${context['x2_' + str(i+1)][j]}));
% endfor
% endif
% endfor
                RectMatrix mat_x2(tmp_x2, length(u1_0));


                std::vector<double*> tmp_ymiss1;

% for i in range(0, len(y1)):
% if (context['bin1_'+str(i+1)] == 'Normal'):
                tmp_ymiss1.push_back(miss${y1[i]});
% endif
% if (context['bin1_'+str(i+1)] == 'Binary'):
                tmp_ymiss1.push_back(misb${y1[i]});
% endif
% if (context['bin1_'+str(i+1)] == 'Ordered'):
                tmp_ymiss1.push_back(miso${y1[i]});
% endif
% if (context['bin1_'+str(i+1)] == 'Unordered'):
% for j in range(0,context['ncats'+y1[i]]-1):
                tmp_ymiss1.push_back(misu${context['ncats'+y1[i]]}_${y1[i]}_${j});
% endfor
% endif
% endfor
                RectMatrix mat_ymiss1(tmp_ymiss1, length(${y1[0]}));

                std::vector<double*> tmp_ymiss2;
% for i in range(0, nvar2):
% if (context['bin2_'+str(i+1)] == 'Normal'):
                tmp_ymiss2.push_back(miss${L2Data}_${y2[i]});
% endif
% if (context['bin2_'+str(i+1)] == 'Binary'):
                tmp_ymiss2.push_back(misb${L2Data}_${y2[i]});
% endif
% if (context['bin2_'+str(i+1)] == 'Ordered'):
                tmp_ymiss2.push_back(miso${L2Data}_${y2[i]});
% endif
% if (context['bin2_'+str(i+1)] == 'Unordered'):
% for j in range(0,context['ncats'+y2[i]]-1):
                tmp_ymiss2.push_back(misu${L2Data}_${context['ncats'+y2[i]]}_${y2[i]}_${j});
% endfor
% endif
% endfor
                RectMatrix mat_ymiss2(tmp_ymiss2, length(u1_0));


                std::vector<double*> tmp_ul1;
% for i in range(0, n1):
                tmp_ul1.push_back(u1_${i});
% endfor
                RectMatrix mat_ul1(tmp_ul1, length(u1_0));

% for var in range(0, nvar1):
% if (context['bin1_'+str(var+1)] == 'Ordered'):
    const int ncats${y1[var]} = ${context['ncats'+y1[var]]};
% endif  
% if (context['bin1_'+str(var+1)] == 'Unordered'):
    const int ncats${y1[var]} = ${context['ncats'+y1[var]]};
% endif  
% endfor

% for var in range(0, nvar2):
% if (context['bin2_'+str(var+1)] == 'Ordered'):
    const int ncats${y2[var]} = ${context['ncats'+y2[var]]};
% endif  
% if (context['bin2_'+str(var+1)] == 'Unordered'):
    const int ncats${y2[var]} = ${context['ncats'+y2[var]]};
% endif  
% endfor



    {
        // Bill's attempt to include ordered cutpoints to go in here (note y step done elsewhere)
% for i in range(0, nvar1):
% if (context['bin1_'+str(i+1)] == 'Ordered'):
        {
            double low[] = ${PyToC([1000] * int(context['ncats'+y1[i]]))};
            double upp[] = ${PyToC([-1000] * int(context['ncats'+y1[i]]))};

            for(int j = 0; j < length(${y1[i]}); j++) {
                if(${y1[i]}[j] <= -9.999e29) {
                } else {            
                    if(miso${y1[i]}[j] < low[int(${y1[i]}[j])])
                        low[int(${y1[i]}[j])] = miso${y1[i]}[j];
                    if(miso${y1[i]}[j] > upp[int(${y1[i]}[j])])
                        upp[int(${y1[i]}[j])] = miso${y1[i]}[j];
                }
            } 
            for(int j = 1; j<ncats${y1[i]}-1; j++) {
                double ul,ll;            
                if(upp[j]>double(alpha${y1[i]}[j-1]))
                    ll = upp[j];
                else 
                    ll = double(alpha${y1[i]}[j-1]);
                if(j == (ncats${y1[i]}-2))
                    ul = low[j+1];
                else if (low[j+1] < double(alpha${y1[i]}[j+1]))
                    ul = low[j+1];
                else
                    ul = double(alpha${y1[i]}[j+1]);
                alpha${y1[i]}[j]= ll+(ul-ll)*random01();
            }
        }
% endif
% endfor
        // Bill's attempt of step for d_e


        // Note currently using a uniform prior for variance matrix
        // Note sb contains all terms needed in MH steps
        SymMatrix sb1(${n1});    

        for(int i = 0; i < length(${y1[0]}); i++) {
<% lenbeta1 = 0 %>\\
<% row = 0 %>\\
<% uind = 0 %>\\
% for i in range(0, nvar1):
% if (context['bin1_'+str(i+1)] == 'Normal'):
            double e${row} = miss${y1[i]}[i] - (${mmult2(context['x1_' + str(i+1)], 'beta1', 'i', lenbeta1)}) - u1_${uind}[${L2ID}[i]];
<% uind += 1 %>\\
<% row += 1 %>\\
% endif
% if (context['bin1_'+str(i+1)] == 'Binary'):
            double e${row} = misb${y1[i]}[i] - (${mmult2(context['x1_' + str(i+1)], 'beta1', 'i', lenbeta1)}) - u1_${uind}[${L2ID}[i]];
<% uind += 1 %>\\
<% row += 1 %>\\
% endif
% if (context['bin1_'+str(i+1)] == 'Ordered'):
            double e${row} = miso${y1[i]}[i] - (${mmult2(context['x1_' + str(i+1)], 'beta1', 'i', lenbeta1)}) - u1_${uind}[${L2ID}[i]];
<% uind += 1 %>\\
<% row += 1 %>\\
% endif
% if (context['bin1_'+str(i+1)] == 'Unordered'):
% for j in range(0,context['ncats'+y1[i]]-1):
            double e${row} = misu${context['ncats'+y1[i]]}_${y1[i]}_${j}[i] - (${mmult2(context['x1_' + str(i+1)+'_'+str(j)], 'beta1', 'i', lenbeta1)}) - u1_${uind}[${L2ID}[i]];
<% uind += 1 %>\\
<% row += 1 %>\\
<% lenbeta1 += len(context['x1_' + str(i + 1) + '_'+ str(j)]) %>\\
% endfor
% else :
<% lenbeta1 += len(context['x1_' + str(i + 1)]) %>\\
% endif
% endfor
% for i in range(0, n1):
% for j in range(i, n1):
            sb1(${i}, ${j}) += e${i} * e${j};
% endfor
% endfor
        }

        bool fail = false;
% if allnormal1:
        int vwe = mat_ymiss1.rows() - ${n1 + 1};
        matrix_sym_invinplace(sb1);
        mat_d_e1 = dwishart(vwe, sb1, fail);
        mat_omega_e1 = matrix_sym_inverse(mat_d_e1, fail);
% else:
        if (runstate == 0)
            mat_d_e1 = matrix_sym_inverse(mat_omega_e1, fail);

        // Now loop over elements of va (omega_e)
        double llcurr;
<% count = 0 %>\\
<% var1 = 0 %>\\
<% withinvar1 = 0 %>\\
% for i in range(0, n1):
<% var2 = var1 %>\\
<% withinvar2 = withinvar1 %>\\
% for j in range(i, n1):
        {
% if var1 == var2 and (context['bin1_'+str(var1+1)] != 'Normal'):
            omega_e1_accept[${count}] = 44;
% else:
            llcurr = length(${y1[0]})*log(matrix_determinant(mat_d_e1))/2;
            for(int i1 = 0; i1 < ${n1}; i1++)
                for(int j1 = 0; j1 < ${n1}; j1++)
                    llcurr -= sb1(i1, j1) * mat_d_e1(i1, j1)/2;
            double old = mat_omega_e1(${i}, ${j});
            mat_omega_e1(${i}, ${j}) = dnorm(mat_omega_e1(${i}, ${j}), 1.0/sqrt(omega_e1_sd[${count}]));
            //SymMatrix temp_d_e1 = matrix_sym_inverse(mat_omega_e1, fail);
            SymMatrix temp_d_e1 = matrix_cholesky(mat_omega_e1, fail);
            // mat_d_e1 = matrix_sym_inverse(mat_omega_e1, fail);
            if (fail == true) {
                mat_omega_e1(${i}, ${j}) = old;
                //mat_d_e1 = matrix_sym_inverse(mat_omega_e1, fail);
            } else {
                temp_d_e1 = matrix_cholinverse(temp_d_e1);
                double llnew = length(${y1[0]})*log(matrix_determinant(temp_d_e1))/2;
                for(int i1=0; i1 < ${n1}; i1++)
                    for(int j1=0; j1 < ${n1}; j1++)
                        llnew -= sb1(i1, j1) * temp_d_e1(i1, j1)/2;
                int change = 1;
                if(isFinite(llnew) == 0)
                    change = 0;
                else if (llnew > llcurr)
                    ;
                else {
                    if(random01() > exp(llnew-llcurr))
                        change = 0;
                }
                if(change == 0) {
                    mat_omega_e1(${i}, ${j}) = old;
                    // mat_d_e1 = matrix_sym_inverse(mat_omega_e1, fail);
                } else {
                    omega_e1_accept[${count}]++;
                    mat_d_e1 = temp_d_e1;
                }
            }
% endif
<% count += 1 %>\\
        }
% if (context['bin1_'+str(var2+1)] != 'Unordered'):
<% var2+= 1 %>\\
% elif withinvar2 != (context['ncats'+y1[var2]]-2):
<% withinvar2 += 1 %>\\
%else:
<% var2+= 1 %>\\
<% withinvar2 = 0 %>\\
% endif
% endfor
% if (context['bin1_'+str(var1+1)] != 'Unordered'):
<% var1+= 1 %>\\
% elif withinvar1 != (context['ncats'+y1[var1]]-2):
<% withinvar1 += 1 %>\\
% else:
<% var1+= 1 %>\\
<% withinvar1 = 0 %>\\
% endif
% endfor

% endif
     
                // Step for beta
                SymMatrix db1(${lenbeta1});
                for (int i = 0; i < length(${y1[0]}); i++) {
                    for (int j1 = 0; j1 < ${lenbeta1}; j1++) {
                        for (int j2 = 0; j2 < j1+1; j2++) {
                            db1(j1, j2) += mat_x1(i, j1) * mat_d_e1(betaind1[j1], betaind1[j2]) * mat_x1(i, j2);
                        }
                    }
                }

                RectMatrix xtdy1(${lenbeta1}, 1);
                for (int i = 0; i < length(${y1[0]}); i++) {
                    for (int j1 = 0; j1 < ${lenbeta1}; j1++) {
                        for (int j2 = 0; j2 < ${n1}; j2++) {
                            xtdy1(j1, 0) += mat_x1(i, j1) * mat_d_e1(betaind1[j1], j2) * (mat_ymiss1(i, j2) - mat_ul1(${L2ID}[i], betaind1[j2]));
                        }
                    }
                }

                fail = false;
                // int err = matrix_sym_invinplace(db1);
                // RectMatrix betahat1 = db1 * xtdy1;
                // mat_beta1 = dmultnormal(betahat1, db1, fail);

                RectMatrix betahat1 = matrix_solve(db1, xtdy1);
                mat_beta1 = dmultnormalinv(betahat1, db1, fail);


		// Step for u (Multivariate!!!!)
                for (int j = 0; j < mat_u1_${i}.rows(); j++) {
                    SymMatrix du1(${n1});
                    for(int j1 = 0;j1 < ${n1}; j1++) {
                        for(int j2 = 0;j2 < ${n1}; j2++) {
                            du1(j1,j2) = mat_d_e1(j1, j2)*idmap_${L2ID}[j].size() + mat_d_u(${n2} + j1, ${n2} + j2);
                        }
                    }
                
                    std::vector<double> sumx(mat_x1.cols());
                    std::vector<double> sumy(mat_ymiss1.cols());
                    for (int id_k = 0; id_k < idmap_${L2ID}[j].size(); id_k++) {
                        int k = idmap_${L2ID}[j][id_k];
                        for (int m = 0; m < mat_x1.cols(); m++) {
                            sumx[m] += mat_x1(k, m);
                        }
                        for (int m = 0; m < mat_ymiss1.cols(); m++) {
                            sumy[m] += mat_ymiss1(k, m);
                        }
                    }

                    std::vector<double> sumymxb(mat_ymiss1.cols());
                    for (int m = 0; m < mat_ymiss1.cols(); m++) {
                        sumymxb[m] = sumy[m];
                    }
                    for (int k = 0; k < mat_x1.cols(); k++) {
                        sumymxb[betaind1[k]] -= mat_beta1(k, 0) * sumx[k]; 
                    }

% if anyl2resp:
                    std::vector<double> sumymxb2(${n2});
                    for (int m = 0; m < mat_ymiss2.cols(); m++) {
                        sumymxb2[m] = mat_ymiss2(j,m);
                    }
                    for (int k = 0; k < mat_x2.cols(); k++) {
                        sumymxb2[betaind2[k]] -= mat_beta2(k, 0) * mat_x2(j,k); 
                    }
% endif

                    RectMatrix xtdu1(${n1}, 1);
                    for(int j1 = 0; j1 < ${n1}; j1++) {
                        for(int j2 = 0;j2 < ${n1}; j2++) {
                            xtdu1(j1, 0) += mat_d_e1(j1, j2) * sumymxb[j2];
                        }
                    }
% if anyl2resp:
                    for(int j1 = 0;j1 < ${n1}; j1++) {
                        for(int j2 = 0;j2 < ${n2}; j2++) {
                            xtdu1(j1, 0) += mat_d_u(${n2}+j1, j2) * sumymxb2[j2];     
                        }
                    }
% endif

                    fail = false;
                    RectMatrix uhat = matrix_solve(du1, xtdu1);
                    RectMatrix mat_u_temp = dmultnormalinv(uhat, du1, fail);

                    for (int j1 = 0; j1 < ${n1}; j1++) {
                        mat_ul1(j, j1) = mat_u_temp(j1, 0);
                    }
		}

% for i in range(0, n1):

<%
mat_x_i = 'mat_x1_' + str(pyyind1[i] + 1)
if (context['bin1_'+str(pyyind1[i]+1)] == 'Unordered'):
    mat_x_i += '_' + str(i - pyymap1[pyyind1[i]])
%>

                for (int j = 0; j < mat_ymiss1.rows(); j++) {
% if (context['bin1_'+str(pyyind1[i]+1)] == 'Normal'):
                    if (mat_y1(j, ${pyyind1[i]}) <= -9.999e29) {
% else:
                    {
% endif
                        double ymissmean = mat_u1_${i}(${L2ID}[j], 0);
                        for (int k = 0; k < ${mat_x_i}.cols(); k++) {
                            ymissmean += mat_beta1(betamap1[${i}] + k, 0) * ${mat_x_i}(j, k);
                        }
                        for (int k = 0; k < mat_ymiss1.cols(); k++) {
                            if (k != ${i}) {
                                ymissmean -= mat_d_e1(${i}, k) * mat_ymiss1(j, k) * (1.0 / mat_d_e1(${i}, ${i}));
                            }
                        }
% for m in range(0, n1):
% if i != m:
<%
mat_x_m = 'mat_x1_' + str(pyyind1[m] + 1)
if (context['bin1_'+str(pyyind1[m]+1)] == 'Unordered'):
    mat_x_m += '_' + str(m - pyymap1[pyyind1[m]])
%>
                        ymissmean += mat_d_e1(${i}, ${m}) * mat_u1_${m}(${L2ID}[j], 0) * (1.0 / mat_d_e1(${i}, ${i}));
                        for (int k = 0; k < ${mat_x_m}.cols(); k++) {
                            ymissmean += mat_d_e1(${i}, ${m}) * (1.0 / mat_d_e1(${i}, ${i})) * mat_beta1(betamap1[${m}] + k, 0) * ${mat_x_m}(j, k);
                        }
% endif
% endfor


% if (context['bin1_'+str(pyyind1[i]+1)] == 'Normal'):
                    if (mat_y1(j, ${pyyind1[i]}) <= -9.999e29) {
                        mat_ymiss1(j, ${i}) = dnorm(ymissmean, mat_d_e1(${i}, ${i}));
                    }
% endif
% if (context['bin1_'+str(pyyind1[i]+1)] == 'Binary'):
                    if (mat_y1(j, ${pyyind1[i]}) <= -9.999e29) 
                        mat_ymiss1(j, ${i}) = dnorm(ymissmean, mat_d_e1(${i}, ${i}));
                    else if(mat_y1(j, ${pyyind1[i]}) <= 0) 
                        mat_ymiss1(j, ${i}) = dtnormal(ymissmean,1/sqrt(mat_d_e1(${i}, ${i})), 2, 0, 0);
                    else          
                        mat_ymiss1(j, ${i}) = dtnormal(ymissmean,1/sqrt(mat_d_e1(${i}, ${i})), 1, 0, 0);
% endif
% if (context['bin1_'+str(pyyind1[i]+1)] == 'Ordered'):
                    if (mat_y1(j, ${pyyind1[i]}) <= -9.999e29) 
                        mat_ymiss1(j, ${i}) = dnorm(ymissmean, mat_d_e1(${i}, ${i}));
                    else if(mat_y1(j, ${pyyind1[i]}) == 0) 
                        mat_ymiss1(j, ${i}) = dtnormal(ymissmean, 1/sqrt(mat_d_e1(${i}, ${i})), 2, 0, 0);
                    else if(mat_y1(j, ${pyyind1[i]}) == ${context['ncats'+y1[pyyind1[i]]] - 1})
                        mat_ymiss1(j, ${i}) = dtnormal(ymissmean, 1/sqrt(mat_d_e1(${i}, ${i})), 1, alpha${y1[pyyind1[i]]}[ncats${y1[pyyind1[i]]}-2], 0);
                    else          
                        mat_ymiss1(j, ${i}) = dtnormal(ymissmean, 1/sqrt(mat_d_e1(${i}, ${i})), 3, alpha${y1[pyyind1[i]]}[int(mat_y1(j, ${pyyind1[i]}))-1], alpha${y1[pyyind1[i]]}[int(mat_y1(j, ${pyyind1[i]}))]);

% endif

% if (context['bin1_'+str(pyyind1[i]+1)] == 'Unordered'):
                    if (mat_y1(j, ${pyyind1[i]}) <= -9.999e29) 
                        mat_ymiss1(j, ${i}) = dnorm(ymissmean, mat_d_e1(${i}, ${i}));
                    else if (mat_y1(j, ${pyyind1[i]}) == ${context['ncats'+y1[pyyind1[i]]] - 1}) 
                        mat_ymiss1(j, ${i}) = dtnormal(ymissmean, 1/sqrt(mat_d_e1(${i}, ${i})), 2, 0, 0);
                    else if (mat_y1(j, ${pyyind1[i]}) == ${i - pyymap1[pyyind1[i]]}) {
                        double limit = 0;
                        for (int k = 0; k < ${context['ncats'+y1[pyyind1[i]]] - 1}; k++) {
                            if ((k != ${i - pyymap1[pyyind1[i]]}) && (mat_ymiss1(j, ${pyymap1[pyyind1[i]]} + k) > limit))
                                limit = mat_ymiss1(j, ${pyymap1[pyyind1[i]]} + k); 
                        }
                        mat_ymiss1(j, ${i}) = dtnormal(ymissmean, 1/sqrt(mat_d_e1(${i}, ${i})), 1, limit, 0);
                    } else {  
                        double limit = mat_ymiss1(j, ${pyymap1[pyyind1[i]]});
                        for (int k = 0; k < ${context['ncats'+y1[pyyind1[i]]] - 1}; k++) {
                            if (mat_ymiss1(j, ${pyymap1[pyyind1[i]]} + k) > limit)
                                limit = mat_ymiss1(j, ${pyymap1[pyyind1[i]]} + k); 
                        }
                        mat_ymiss1(j, ${i}) = dtnormal(ymissmean, 1/sqrt(mat_d_e1(${i}, ${i})), 2, 0, limit);
                    }

% endif

                    }
                }
% endfor






















               // Level 2 code

    {
        // Bill's attempt to include ordered cutpoints to go in here (note y step done elsewhere)
% for i in range(0, nvar2):
% if (context['bin2_'+str(i+1)] == 'Ordered'):
        {
            double low[] = ${PyToC([1000] * int(context['ncats'+y2[i]]))};
            double upp[] = ${PyToC([-1000] * int(context['ncats'+y2[i]]))};

            for(int j = 0; j < length(${L2Data}_${y2[i]}); j++) {
                if (${L2Data}_${y2[i]}[j] <= -9.999e29) {
                } else {
                    if(miso${L2Data}_${y2[i]}[j] < low[int(${L2Data}_${y2[i]}[j])])
                        low[int(${L2Data}_${y2[i]}[j])] = miso${L2Data}_${y2[i]}[j];
                    if(miso${L2Data}_${y2[i]}[j] > upp[int(${L2Data}_${y2[i]}[j])])
                        upp[int(${L2Data}_${y2[i]}[j])] = miso${L2Data}_${y2[i]}[j];
                }
            }

            for(int j = 1; j < ncats${y2[i]}-1; j++) {
                double ul, ll;            
                if(upp[j]>double(alpha${y2[i]}[j-1]))
                    ll = upp[j];
                else 
                    ll = double(alpha${y2[i]}[j-1]);
                if(j == (ncats${y2[i]}-2))
                    ul = low[j+1];
                else if (low[j+1] < double(alpha${y2[i]}[j+1]))
                    ul = low[j+1];
                else
                    ul = double(alpha${y2[i]}[j+1]);
                alpha${y2[i]}[j]= ll + (ul-ll) * random01();
            }
        }
% endif
% endfor
        // Bill's attempt of step for d_u
                   
        // Note currently using a uniform prior for variance matrix
        // Note sb contains all terms needed in MH steps
        SymMatrix sb2(${nr2});
        for(int i = 0; i < length(u1_0); i++) {
<% lenbeta2 = 0 %>\\
<% row = 0 %>\\
% for i in range(0, nvar2):
% if (context['bin2_'+str(i+1)] == 'Normal'):
            double e${row} = miss${L2Data}_${y2[i]}[i] - (${mmult3(L2Data, context['x2_' + str(i+1)], 'beta2', 'i', lenbeta2)});
<% row += 1 %>\\
% endif
% if (context['bin2_'+str(i+1)] == 'Binary'):
            double e${row} = misb${L2Data}_${y2[i]}[i] - (${mmult3(L2Data, context['x2_' + str(i+1)], 'beta2', 'i', lenbeta2)});
<% row += 1 %>\\
% endif
% if (context['bin2_'+str(i+1)] == 'Ordered'):
            double e${row} = miso${L2Data}_${y2[i]}[i] - (${mmult3(L2Data, context['x2_' + str(i+1)], 'beta2', 'i', lenbeta2)});
<% row += 1 %>\\
% endif
% if (context['bin2_'+str(i+1)] == 'Unordered'):
% for j in range(0,context['ncats'+y2[i]]-1):
            double e${row} = misu${L2Data}_${context['ncats'+y2[i]]}_${y2[i]}_${j}[i] - (${mmult3(L2Data, context['x2_' + str(i+1)+'_'+str(j)], 'beta2', 'i', lenbeta2)}); 
<% row += 1 %>\\
<% lenbeta2 += len(context['x2_' + str(i + 1) + '_'+ str(j)]) %>\\
% endfor
% else :
<% lenbeta2 += len(context['x2_' + str(i + 1)]) %>\\
% endif
% endfor

% for i in range(0, n1):
            double e${row} = u1_${i}[i];
<% row += 1 %>\\
% endfor

% for i in range(0, nr2):
% for j in range(i, nr2):
            sb2(${i}, ${j}) += e${i} * e${j};
% endfor
% endfor
        }

        bool fail = false;
% if allnormal2:
        int vwu = mat_ymiss2.rows() - ${nr2 + 1};
        matrix_sym_invinplace(sb2);
        mat_d_u = dwishart(vwu, sb2, fail);
        mat_omega_u = matrix_sym_inverse(mat_d_u, fail);
% else:
        if (runstate == 0)
            mat_d_u = matrix_sym_inverse(mat_omega_u, fail);
        // Now loop over elements of va (omega_e)
<% count = 0 %>\\
<% var1 = 0 %>\\
<% withinvar1 = 0 %>\\
% for i in range(0, nr2):
<% var2 = var1 %>\\
% if var1 >= len(y2):
<% var2 = var1 + 1%>
% endif
<% withinvar2 = withinvar1 %>\\
% for j in range(i, nr2):
        {
% if var1 == var2 and (context['bin2_'+str(var1+1)] != 'Normal'):
            omega_u_accept[${count}] = 44;
% else:
            double llcurr = length(u1_0)*log(matrix_determinant(mat_d_u))/2;
            for(int i1=0; i1<${nr2}; i1++)
                for(int j1=0; j1<${nr2}; j1++)
                    llcurr -= sb2(i1, j1) * mat_d_u(i1, j1)/2;
            double old = mat_omega_u(${i}, ${j});
            mat_omega_u(${i}, ${j}) = dnorm(mat_omega_u(${i}, ${j}), 1.0/sqrt(omega_u_sd[${count}]));
            // mat_d_u = matrix_sym_inverse(mat_omega_u, fail);
            SymMatrix temp_d_u = matrix_cholesky(mat_omega_u, fail);
            if (fail == true) {
                mat_omega_u(${i}, ${j}) = old;
                //mat_d_u = matrix_sym_inverse(mat_omega_u, fail);
            } else {
                temp_d_u = matrix_cholinverse(temp_d_u);
                double llnew = length(u1_0)*log(matrix_determinant(temp_d_u))/2;
                for(int i1=0; i1<${nr2}; i1++)
                    for(int j1=0; j1<${nr2}; j1++)
                        llnew -= sb2(i1, j1) * temp_d_u(i1, j1)/2;
                int change = 1;
                if(isFinite(llnew) == 0)
                    change = 0;
                else if (llnew > llcurr)
                    ;
                else {
                    if(random01() > exp(llnew - llcurr))
                        change = 0;
                }
                if(change == 0) {
                    mat_omega_u(${i}, ${j}) = old;
                    // mat_d_u = matrix_sym_inverse(mat_omega_u, fail);
                } else {
                    omega_u_accept[${count}]++;
                    mat_d_u = temp_d_u;
                }
            }
% endif
<% count += 1 %>\\
        }
% if var2 < len(y2):

% if (context['bin2_'+str(var2+1)] != 'Unordered'):
<% var2+= 1 %>\\
% elif withinvar2 != (context['ncats'+y2[var2]]-2):
<% withinvar2 += 1 %>\\
% else:
<% var2+= 1 %>\\
<% withinvar2 = 0 %>\\
% endif

% endif

% endfor

% if var1 < len(y2):

% if (context['bin2_'+str(var1+1)] != 'Unordered'):
<% var1+= 1 %>\\
% elif withinvar1 != (context['ncats'+y2[var1]]-2):
<% withinvar1 += 1 %>\\
% else:
<% var1+= 1 %>\\
<% withinvar1 = 0 %>\\
% endif

% endif

% endfor


% endif


% if anyl2resp:
% if condmarg:
                // Calculate submatrix of omega_u and invert
		SymMatrix mat_omega_beta2(omega_u, ${n2});
                SymMatrix mat_d_beta2 = matrix_sym_inverse(mat_omega_beta2, fail);
% endif

                // Step for beta

                SymMatrix db2(${lenbeta2});
                for (int i = 0; i < length(${L2Data}_${y2[0]}); i++) {
                    for (int j1 = 0; j1 < ${lenbeta2}; j1++) {
                        for (int j2 = 0; j2 < j1+1; j2++) {
% if condmarg:
                            db2(j1, j2) += mat_x2(i, j1) * mat_d_beta2(betaind2[j1], betaind2[j2]) * mat_x2(i, j2);
% else:
                            db2(j1, j2) += mat_x2(i, j1) * mat_d_u(betaind2[j1], betaind2[j2]) * mat_x2(i, j2);
% endif
                        }
                    }
                }

                RectMatrix xtdy2(${lenbeta2}, 1);
                for (int i = 0; i < length(${L2Data}_${y2[0]}); i++) {
                    for (int j1 = 0; j1 < ${lenbeta2}; j1++) {
                        for (int j2 = 0; j2 < ${n2}; j2++) {
% if condmarg:
                            xtdy2(j1, 0) += mat_x2(i, j1) * mat_d_beta2(betaind2[j1], j2) * mat_ymiss2(i, j2);
% else:
                            xtdy2(j1, 0) += mat_x2(i, j1) * mat_d_u(betaind2[j1], j2) * mat_ymiss2(i, j2);
% endif
                        }
% if not condmarg:
                        for (int j3 = 0; j3 < ${n1}; j3++) {
                             xtdy2(j1, 0) += mat_d_u(betaind2[j1], j3 + ${n2}) * mat_ul1(i, j3);
                        }
% endif
                    }
                }

                fail = false;
                // int err = matrix_sym_invinplace(db2);
                // RectMatrix betahat2 = db2 * xtdy2;
                // mat_beta2 = dmultnormal(betahat2, db2, fail);

                RectMatrix betahat2 = matrix_solve(db2, xtdy2);
                mat_beta2 = dmultnormalinv(betahat2, db2, fail);


% for i in range(0, n2):

<%
mat_x_i = 'mat_x2_' + str(pyyind2[i] + 1)
if (context['bin2_'+str(pyyind2[i]+1)] == 'Unordered'):
    mat_x_i += '_' + str(i - pyymap2[pyyind2[i]])
%>

                for (int j = 0; j < mat_ymiss2.rows(); j++) {
                    double tmp = 0.0;
% if (context['bin2_'+str(pyyind2[i]+1)] == 'Normal'):
                    if (mat_y2(j, ${pyyind2[i]}) <= -9.999e29) {
% else:
                    {
% endif
                        for (int k = 0; k < ${mat_x_i}.cols(); k++) {
                            tmp += mat_beta2(betamap2[${i}] + k, 0) * ${mat_x_i}(j, k);
                        }
                        for (int k = 0; k < mat_ymiss2.cols(); k++) {
                            if (k != ${i}) {
                                tmp -= mat_d_u(${i}, k) * mat_ymiss2(j, k) * (1.0 / mat_d_u(${i}, ${i}));
                            }
                        }
% for m in range(0, n2):
% if i != m:
<%
mat_x_m = 'mat_x2_' + str(pyyind2[m] + 1)
if (context['bin2_'+str(pyyind2[m]+1)] == 'Unordered'):
    mat_x_m += '_' + str(m - pyymap2[pyyind2[m]])
%>
                        for (int k = 0; k < ${mat_x_m}.cols(); k++) {
                            tmp += mat_d_u(${i}, ${m}) * (1 / mat_d_u(${i}, ${i})) * mat_beta2(betamap2[${m}] + k, 0) * ${mat_x_m}(j, k);
                        }
% endif
% endfor

% for m in range(0, n1):
                        tmp -= mat_d_u(${i}, ${n2+m}) * (1 / mat_d_u(${i}, ${i})) * mat_ul1(j, ${m});
% endfor


% if (context['bin2_'+str(pyyind2[i]+1)] == 'Normal'):
                    if (mat_y2(j, ${pyyind2[i]}) <= -9.999e29) {
                        mat_ymiss2(j, ${i}) = dnorm(tmp, mat_d_u(${i}, ${i}));
                    }
% endif
% if (context['bin2_'+str(pyyind2[i]+1)] == 'Binary'):
                    if (mat_y2(j, ${pyyind2[i]}) <= -9.999e29) 
                        mat_ymiss2(j, ${i}) = dnorm(tmp, mat_d_u(${i}, ${i}));
                    else if(mat_y2(j, ${pyyind2[i]}) <= 0) 
                        mat_ymiss2(j, ${i}) = dtnormal(tmp,1/sqrt(mat_d_u(${i}, ${i})), 2, 0, 0);
                    else          
                        mat_ymiss2(j, ${i}) = dtnormal(tmp,1/sqrt(mat_d_u(${i}, ${i})), 1, 0, 0);
% endif
% if (context['bin2_'+str(pyyind2[i]+1)] == 'Ordered'):
                    if (mat_y2(j, ${pyyind2[i]}) <= -9.999e29) 
                        mat_ymiss2(j, ${i}) = dnorm(tmp, mat_d_u(${i}, ${i}));
                    else if(mat_y2(j, ${pyyind2[i]}) == 0) 
                        mat_ymiss2(j, ${i}) = dtnormal(tmp, 1/sqrt(mat_d_u(${i}, ${i})), 2, 0, 0);
                    else if(mat_y2(j, ${pyyind2[i]}) == ${context['ncats'+y2[pyyind2[i]]] - 1})
                        mat_ymiss2(j, ${i}) = dtnormal(tmp, 1/sqrt(mat_d_u(${i}, ${i})), 1, alpha${y2[pyyind2[i]]}[ncats${y2[pyyind2[i]]}-2], 0);
                    else          
                        mat_ymiss2(j, ${i}) = dtnormal(tmp, 1/sqrt(mat_d_u(${i}, ${i})), 3, alpha${y2[pyyind2[i]]}[int(mat_y2(j, ${pyyind1[i]}))-1], alpha${y2[pyyind2[i]]}[int(mat_y2(j, ${pyyind1[i]}))]);

% endif

% if (context['bin2_'+str(pyyind2[i]+1)] == 'Unordered'):
                    if (mat_y2(j, ${pyyind2[i]}) <= -9.999e29) {
                        mat_ymiss2(j, ${i}) = dnorm(tmp, mat_d_u(${i}, ${i}));
                    } else if (mat_y2(j, ${pyyind2[i]}) == ${context['ncats'+y2[pyyind2[i]]] - 1}) {
                        mat_ymiss2(j, ${i}) = dtnormal(tmp, 1/sqrt(mat_d_u(${i}, ${i})), 2, 0, 0);
                    } else if (mat_y2(j, ${pyyind2[i]}) == ${i - pyymap2[pyyind2[i]]}) {
                        double limit = 0;
                        for (int k = 0; k < ${context['ncats'+y2[pyyind2[i]]] - 1}; k++) {
                            if ((k != ${i - pyymap2[pyyind2[i]]}) && (mat_ymiss2(j, ${pyymap2[pyyind2[i]]} + k) > limit))
                                limit = mat_ymiss2(j, ${pyymap2[pyyind2[i]]} + k); 
                        }
                        mat_ymiss2(j, ${i}) = dtnormal(tmp, 1/sqrt(mat_d_u(${i}, ${i})), 1, limit, 0);
                    } else {  
                        double limit = mat_ymiss2(j, ${pyymap2[pyyind2[i]]});
                        for (int k = 0; k < ${context['ncats'+y2[pyyind2[i]]] - 1}; k++) {
                            if (mat_ymiss2(j, ${pyymap2[pyyind2[i]]} + k) > limit)
                                limit = mat_ymiss2(j, ${pyymap2[pyyind2[i]]} + k); 
                        }
                        mat_ymiss2(j, ${i}) = dtnormal(tmp, 1/sqrt(mat_d_u(${i}, ${i})), 2, 0, limit);
                    }

% endif

                    }
                }
% endfor

% endif

        }

    }
'''

    preparedata = '''
l1data = data['datafile']

for i in range(0, len(y1)):
    if (context['bin1_'+str(i+1)] == 'Normal'):
        context['miss'+y1[i]].ncols = -1*len(l1data.variables[y1[i]]['data'])
    if (context['bin1_'+str(i+1)] == 'Binary'):
        context['misb'+y1[i]].ncols = -1*len(l1data.variables[y1[i]]['data'])
    if (context['bin1_'+str(i+1)] == 'Ordered'):
        context['miso'+y1[i]].ncols = -1*len(l1data.variables[y1[i]]['data'])
    if (context['bin1_'+str(i+1)] == 'Unordered'):
        for j in range(0, context['ncats'+y1[i]]-1):
            context['misu'+str(context['ncats'+y1[i]])+'_'+y1[i]+'_'+str(j)].ncols = -1*len(l1data.variables[y1[i]]['data'])
        context['origv1_'+str(i+1)].name.append(y1[i]) 
        for var in context['origv1_'+str(i+1)]:
            uniqvals = list(set(l1data.variables[var]['data'].compressed()))
            uniqvals.sort()
            uniqvals.remove(uniqvals[len(uniqvals)-1])
            for j in range(0, len(uniqvals)):
                l1data.addvariable(var + '_' + str(int(j)), data = (l1data.variables[var]['data'] == uniqvals[j]).astype(float))
                context['y1_'+str(i+1)].name.append(var + '_' + str(int(j)))

if anyl2resp:
    l2data = data[str(L2Data)]
    for i in range(0, len(y2)):
        if (context['bin2_'+str(i+1)] == 'Normal'):
            context['miss'+str(L2Data)+'_'+y2[i]].ncols = -1*len(l2data.variables[y2[i]]['data'])
        if (context['bin2_'+str(i+1)] == 'Binary'):
            context['misb'+str(L2Data)+'_'+y2[i]].ncols = -1*len(l2data.variables[y2[i]]['data'])
        if (context['bin2_'+str(i+1)] == 'Ordered'):
            context['miso'+str(L2Data)+'_'+y2[i]].ncols = -1*len(l2data.variables[y2[i]]['data'])
        if (context['bin2_'+str(i+1)] == 'Unordered'):
            for j in range(0, context['ncats'+y2[i]]-1):
                context['misu'+str(L2Data)+'_'+str(context['ncats'+y2[i]])+'_'+y2[i]+'_'+str(j)].ncols = -1*len(l2data.variables[y2[i]]['data'])
            context['origv2_'+str(i+1)].name.append(y2[i]) 
            for var in context['origv2_'+str(i+1)]:
                uniqvals = list(set(l2data.variables[var]['data'].compressed()))
                uniqvals.sort()
                uniqvals.remove(uniqvals[len(uniqvals)-1])
                for j in range(0, len(uniqvals)):
                    l2data.addvariable(var + '_' + str(int(j)), data = (l2data.variables[var]['data'] == uniqvals[j]).astype(float))
                    context['y2_'+str(i+1)].name.append(var + '_' + str(int(j)))
'''
